#include <stdlib.h>
#include <stdio.h>
#include "mpi.h"

int main(int argc, char* argv[]) 
{
	int		procs, myid;
	int		i;
	double	*vectorA, *vectorB;
	double  *myvectorA, *myvectorB;
	double	dotProduct;
	int		vectorSize = 80;
	int 	scatterSize;

	MPI_Init(&argc, &argv); 
	MPI_Comm_rank(MPI_COMM_WORLD, &myid);
	MPI_Comm_size(MPI_COMM_WORLD, &procs);

	vectorA = NULL;
	vectorB = NULL;
	if(myid == 0)
	{
		vectorA  = (double *) malloc(vectorSize*sizeof(double));
		vectorB  = (double *) malloc(vectorSize*sizeof(double));
		for(i=0; i<vectorSize; i++)
		{
			vectorA[i] = 1.0;
			vectorB[i] = 1.0;
		}
	}

	if((vectorSize < procs) || (vectorSize % procs != 0)) 
	{
		MPI_Finalize();
		if(myid == 0)
			printf("Število elementov ni večkratnih procesov...\n");

		return 1;
	}  	

	scatterSize = vectorSize / procs;
	myvectorA = (double *)malloc( scatterSize*sizeof(double));
	myvectorB = (double *)malloc( scatterSize*sizeof(double));

	MPI_Scatter(vectorA, scatterSize, MPI_DOUBLE, 
				myvectorA, scatterSize, MPI_DOUBLE, 0, MPI_COMM_WORLD);
	MPI_Scatter(vectorB, scatterSize, MPI_DOUBLE, 
				myvectorB, scatterSize, MPI_DOUBLE, 0, MPI_COMM_WORLD);

	double mydotProduct = 0.0;
	for(i=0; i<scatterSize; i++) 
		mydotProduct += (myvectorA[i] * myvectorB[i]);

	double *mydotProducts;

	mydotProducts = NULL;
	if (myid == 0)
		mydotProducts = (double *)malloc(procs * sizeof(double));
	
	MPI_Gather(&mydotProduct, 1, MPI_DOUBLE, mydotProducts, 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);

	if (myid == 0)
	{
		dotProduct = 0.0;
		for (int i=0; i<procs; i++)
			dotProduct += mydotProducts[i];
		printf("Skalarni produkt: %f\n", dotProduct);
	}

	free(myvectorA);
	free(myvectorB);
	free(mydotProducts);
	free(vectorA);
	free(vectorB);

	MPI_Finalize();

	return 0;
}
